@@ -1338,5 +1338,239 @@ class emailmgmt {
         
         return ['status' => '1', 'msg' => 'Failed to create DKIM records after maximum retries'];
     }
+    
+    /**
+     * Import existing email accounts from mail server (root users only)
+     * @param string $domain Domain to import accounts from
+     * @return array Result with status and imported accounts
+     */
+    public function importEmailAccountsFromMailServer($domain) {
+        // Only root users can import email accounts
+        if (AUTH_USER !== 'root') {
+            return ['status' => '1', 'msg' => 'Only root users can import email accounts'];
+        }
+        
+        if (!$this->mail_client) {
+            return ['status' => '1', 'msg' => 'Mail server client not configured'];
+        }
+        
+        if (!$this->db) {
+            return ['status' => '1', 'msg' => 'Database connection failed'];
+        }
+        
+        // Validate that the domain exists on this server
+        try {
+            $stmt = $this->db->prepare("SELECT username FROM whp.domains WHERE domain_name = ? AND active = 1");
+            $stmt->execute([$domain]);
+            $domain_info = $stmt->fetch(PDO::FETCH_ASSOC);
+            
+            if (!$domain_info) {
+                return ['status' => '1', 'msg' => 'Domain not found on this server'];
+            }
+            
+            $domain_owner = $domain_info['username'];
+            
+        } catch (Exception $e) {
+            error_log("Failed to validate domain: " . $e->getMessage());
+            return ['status' => '1', 'msg' => 'Failed to validate domain'];
+        }
+        
+        try {
+            // Fetch email accounts from mail server
+            $response = $this->mail_client->getEmailAccounts($domain);
+            
+            if (!isset($response['results'])) {
+                return ['status' => '1', 'msg' => 'Invalid response from mail server'];
+            }
+            
+            $imported_count = 0;
+            $skipped_count = 0;
+            $failed_count = 0;
+            $imported_accounts = [];
+            $errors = [];
+            
+            foreach ($response['results'] as $account) {
+                // Skip if it's a forwarder-only account (no actual mailbox)
+                if ($account['redirect_only'] === true && !empty($account['redirect_to'])) {
+                    // Import as forwarder
+                    $result = $this->importEmailForwarderFromMailServer($account, $domain_owner);
+                    if ($result['status'] === '0') {
+                        $imported_count++;
+                    } else {
+                        $failed_count++;
+                        $errors[] = "Forwarder {$account['address']}: " . $result['msg'];
+                    }
+                    continue;
+                }
+                
+                // Skip disabled accounts unless they're actual mailboxes
+                if ($account['disabled'] === true && $account['redirect_only'] === true) {
+                    $skipped_count++;
+                    continue;
+                }
+                
+                // Check if account already exists in our database
+                try {
+                    $stmt = $this->db->prepare("SELECT id FROM whp.email_accounts WHERE email_address = ?");
+                    $stmt->execute([$account['address']]);
+                    if ($stmt->fetch()) {
+                        $skipped_count++;
+                        continue; // Account already exists
+                    }
+                } catch (Exception $e) {
+                    error_log("Failed to check existing account: " . $e->getMessage());
+                    $failed_count++;
+                    continue;
+                }
+                
+                // Import the email account
+                $result = $this->importEmailAccount($account, $domain_owner);
+                if ($result['status'] === '0') {
+                    $imported_count++;
+                    $imported_accounts[] = $account['address'];
+                } else {
+                    $failed_count++;
+                    $errors[] = "{$account['address']}: " . $result['msg'];
+                }
+            }
+            
+            $message = "Import complete: $imported_count imported, $skipped_count skipped, $failed_count failed";
+            if (!empty($errors)) {
+                $message .= "\nErrors: " . implode("; ", array_slice($errors, 0, 5));
+                if (count($errors) > 5) {
+                    $message .= " and " . (count($errors) - 5) . " more...";
+                }
+            }
+            
+            return [
+                'status' => '0',
+                'msg' => $message,
+                'imported' => $imported_accounts,
+                'imported_count' => $imported_count,
+                'skipped_count' => $skipped_count,
+                'failed_count' => $failed_count
+            ];
+            
+        } catch (Exception $e) {
+            error_log("Failed to import email accounts: " . $e->getMessage());
+            return ['status' => '1', 'msg' => 'Failed to fetch accounts from mail server: ' . $e->getMessage()];
+        }
+    }
+    
+    /**
+     * Import a single email account from mail server data
+     * @param array $account Account data from mail server
+     * @param string $username Owner username
+     * @return array Result with status
+     */
+    private function importEmailAccount($account, $username) {
+        try {
+            // Extract domain from email address
+            $email_parts = explode('@', $account['address']);
+            if (count($email_parts) !== 2) {
+                return ['status' => '1', 'msg' => 'Invalid email address format'];
+            }
+            $domain = $email_parts[1];
+            
+            // Generate a random password (user will need to reset it)
+            $temp_password = bin2hex(random_bytes(16));
+            
+            // Get storage quota from mail server if available
+            $quota_mb = 100; // Default quota
+            
+            // Insert into database
+            $stmt = $this->db->prepare("
+                INSERT INTO whp.email_accounts 
+                (email_address, username, domain_name, password_hash, quota_mb, active, created_at, updated_at)
+                VALUES (?, ?, ?, ?, ?, ?, NOW(), NOW())
+            ");
+            
+            $stmt->execute([
+                $account['address'],
+                $username,
+                $domain,
+                password_hash($temp_password, PASSWORD_DEFAULT),
+                $quota_mb,
+                !$account['disabled'] // Convert disabled to active
+            ]);
+            
+            return ['status' => '0', 'msg' => 'Account imported successfully'];
+            
+        } catch (Exception $e) {
+            error_log("Failed to import email account {$account['address']}: " . $e->getMessage());
+            return ['status' => '1', 'msg' => 'Failed to import account: ' . $e->getMessage()];
+        }
+    }
+    
+    /**
+     * Import email forwarder from mail server data
+     * @param array $forwarder Forwarder data from mail server
+     * @param string $username Owner username
+     * @return array Result with status
+     */
+    private function importEmailForwarderFromMailServer($forwarder, $username) {
+        try {
+            // Check if forwarder already exists
+            $stmt = $this->db->prepare("SELECT id FROM whp.email_forwarders WHERE source_email = ?");
+            $stmt->execute([$forwarder['address']]);
+            if ($stmt->fetch()) {
+                return ['status' => '1', 'msg' => 'Forwarder already exists'];
+            }
+            
+            // Import each redirect_to address
+            foreach ($forwarder['redirect_to'] as $destination) {
+                $stmt = $this->db->prepare("
+                    INSERT INTO whp.email_forwarders 
+                    (source_email, destination_email, destination_type, username, active, created_at, updated_at)
+                    VALUES (?, ?, 'external', ?, ?, NOW(), NOW())
+                ");
+                
+                $stmt->execute([
+                    $forwarder['address'],
+                    $destination,
+                    $username,
+                    !$forwarder['disabled']
+                ]);
+            }
+            
+            return ['status' => '0', 'msg' => 'Forwarder imported successfully'];
+            
+        } catch (Exception $e) {
+            error_log("Failed to import forwarder {$forwarder['address']}: " . $e->getMessage());
+            return ['status' => '1', 'msg' => 'Failed to import forwarder: ' . $e->getMessage()];
+        }
+    }
+    
+    /**
+     * Get list of domains available for import (root only)
+     * @return array Result with domains that have email accounts on mail server
+     */
+    public function getDomainsAvailableForImport() {
+        if (AUTH_USER !== 'root') {
+            return ['status' => '1', 'msg' => 'Only root users can access this function'];
+        }
+        
+        if (!$this->db) {
+            return ['status' => '1', 'msg' => 'Database connection failed'];
+        }
+        
+        try {
+            // Get all active domains on this server
+            $stmt = $this->db->prepare("
+                SELECT domain_name, username 
+                FROM whp.domains 
+                WHERE active = 1 
+                ORDER BY domain_name
+            ");
+            $stmt->execute();
+            $domains = $stmt->fetchAll(PDO::FETCH_ASSOC);
+            
+            return ['status' => '0', 'domains' => $domains];
+            
+        } catch (Exception $e) {
+            error_log("Failed to get domains for import: " . $e->getMessage());
+            return ['status' => '1', 'msg' => 'Failed to retrieve domains'];
+        }
+    }
 }
 ?> 
\ No newline at end of file
